/* Row Constructors */
-- table definintion
CREATE TABLE UserTypes (
	UserTypeId int IDENTITY(1,1) NOT NULL,
	UserType varchar(20) NOT NULL
)
GO

-- typical INSERT statements
INSERT INTO UserTypes (UserType)
VALUES ('User')
INSERT INTO UserTypes (UserType)
VALUES ('Manager')
INSERT INTO UserTypes (UserType)
VALUES ('Admin')

-- shorter 'cheating' syntax works fine and technically correct
-- but suffers from being less read-able
INSERT INTO UserTypes (UserType)
SELECT 'User'
UNION SELECT 'Manager'
UNION SELECT 'Admin'

-- Row Constructor Approach:
INSERT INTO UserTypes (UserType)
VALUES 
('User'),
('Manager'),
('Admin')



/* Inline Variable Assigment   */
-- old approach (correct)
DECLARE @value int
SET @value = 27

-- incorrect/bad approach 
--  (worked in previous versions but not now)
DECLARE @value2 int
SELECT @value = -1  -- Use SET instead of 'SELECT'

SELECT @value, @value2

-- proper use of SELECT when assigning variables:
DECLARE @count int
SELECT @count = COUNT(*) FROM Videos -- SELECT = assign

SELECT @count  -- SELECT here = ... well... SELECT

-- inline variable assignment
DECLARE @today date = GETDATE()
SELECT @today

-- but note that this doesn't work:
DECLARE @never int = SELECT COUNT(*) FROM Videos
SELECT @never


/* Compound Assignment Operators */
DECLARE @int int = 5
DECLARE @string varchar(200) = 'some string data'

-- pre SQL Server 2008 assignment:
SET @int = @int + 12
SET @string = @string + ' (pretty lame eh?)'

SELECT @int, @string

-- Compound Assignment Operators:
SET @int += 12
SET @string += ' (not so lame)'

SELECT @int, @string


/* Grouping Sets */
-- simple GROUP BY / ROLLUP:
SELECT
	COUNT(v.VideoId) [Videos],
	s.Name
FROM
	Videos v 
	INNER JOIN Series s ON v.SeriesId = s.SeriesId
GROUP BY
	s.Name
	WITH CUBE
	
-- a bit more complex:
DECLARE @cutoff date = DATEADD(day, -10, GETDATE())

SELECT
	VideoName,
	Series,
	[Type],
	COUNT(VideoName) [Impressions]	
FROM
	VideoImpressions
WHERE
	[TimeStamp] > @cutoff
GROUP BY
	VideoName,
	Series,
	[Type]
	WITH ROLLUP

-- an example of using a GROUPING SETS
DECLARE @cutoff date = DATEADD(day, -10, GETDATE())

SELECT
	VideoName,
	Series,
	[Type],
	COUNT(VideoName) [Impressions]	
FROM
	VideoImpressions
WHERE
	[TimeStamp] > @cutoff
GROUP BY
	-- NOTE that the groupings and order of elements and groups 
	-- drastically change summarization/output:
	GROUPING SETS( 
		(VideoName, Series), (Type)
	) 

-- See Books Online for more concrete examples:
http://technet.microsoft.com/en-us/library/bb522495.aspx


/* Compostable/Nested DML enhancements for OUTPUT clause */

DECLARE @names TABLE (
	NameId int IDENTITY NOT NULL, 
	Name varchar(20) NOT NULL 
)

INSERT INTO @names (Name)
SELECT o.Name FROM
(
	INSERT INTO @names (Name)
	OUTPUT inserted.Name -- new compostable DML goodness
	VALUES 
	('Thomas'),
	('Sandra'),
	('Vijay'),
	('Isaac'),
	('Lars')
) AS o

-- note the order of entered items
SELECT * FROM @names


/* Simple MERGE statement */

DECLARE @jellybeans TABLE (
	ID int IDENTITY NOT NULL, 
	Flavor varchar(20) NOT NULL, 
	Number int
)

INSERT INTO @jellybeans (Flavor, Number)
VALUES ('Green', 20), ('Red', 18),
	('Popcorn', 12), ('Ear Wax', 3)
	
-- another table 'full' of jellybeans:
DECLARE @jellybellies TABLE (
	Flavor varchar(20) NOT NULL, 
	Number int
)

INSERT INTO @jellybellies (Flavor, Number)
VALUES ('Red', 22),('Grape', 12)

MERGE INTO @jellybeans j1
USING @jellybellies j2
ON 
 (j1.Flavor = j2.Flavor)
WHEN MATCHED THEN 
	UPDATE 
		SET Number += j2.Number
WHEN NOT MATCHED THEN 
	INSERT (Flavor, Number)
	VALUES (j2.Flavor, j2.Number)
;
	
SELECT * FROM @jellybeans